/*
 *  freesmartphone-phonebook.h
 *
 *  Copyright(C) 2008 Ixonos Plc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
 */

/*
 * Written by
 *   Heikki Paajanen
 *   Vesa Pikki
 */

#ifndef FREESMARTPHONE_SIM_H
#define FREESMARTPHONE_SIM_H

#include <glib.h>
#include <dbus/dbus-glib.h>

#include "freesmartphone.h"

typedef void (*FSGSMPhonebookGetStorageSpacesReplyFunc) (gchar **spaces,
                                                         GError *error,
                                                         gpointer userdata );

typedef void (*FSGSMPhonebookGetDefaultStorageSpaceReplyFunc) (gchar *spaces,
                                                               GError *error,
                                                               gpointer userdata );
typedef void (*FSGSMPhonebookReplyFunc) (GError *error,
                                         gpointer userdata );

typedef void (*FSGSMPhonebookGetIndexBoundariesReplyFunc) (gint lowest_index,
                                                           gint highest_index,
                                                           GError *error,
                                                           gpointer userdata );

typedef void (*FSGSMPhonebookRetrieveEntryReplyFunc) (gint type,
                                                      gchar *name,
                                                      gchar *contents,
                                                      GError *error,
                                                      gpointer userdata );

gboolean fso_gsm_phonebook_get_storage_spaces(FreeSmartphone *fs,
                                              FSGSMPhonebookGetStorageSpacesReplyFunc cb,
                                              gpointer userdata);

gboolean fso_gsm_phonebook_get_default_storage_space(FreeSmartphone *fs,
                                                     FSGSMPhonebookGetDefaultStorageSpaceReplyFunc cb,
                                                     gpointer userdata);
gboolean fso_gsm_phonebook_set_default_storage_space(FreeSmartphone *fs,
                                                     gchar *space,
                                                     FSGSMPhonebookReplyFunc cb,
                                                     gpointer userdata);
gboolean fso_gsm_phonebook_get_index_boundaries(FreeSmartphone *fs,
                                                FSGSMPhonebookGetIndexBoundariesReplyFunc cb,
                                                gpointer userdata);
gboolean fso_gsm_phonebook_delete_entry(FreeSmartphone *fs,
                                        gint index,
                                        FSGSMPhonebookReplyFunc cb,
                                        gpointer userdata);
gboolean fso_gsm_phonebook_store_entry(FreeSmartphone *fs,
                                       gint index,
                                       gint type,
                                       gchar *name,
                                       gchar *contents,
                                       FSGSMPhonebookReplyFunc cb,
                                       gpointer userdata);

gboolean fso_gsm_phonebook_retrieve_entry(FreeSmartphone *fs,
                                          gint index,
                                          FSGSMPhonebookRetrieveEntryReplyFunc cb,
                                          gpointer userdata);

#endif /* FREESMARTPHONE_PHONEBOOK_H */

