/*
 *  dbus_phonebook_object.c
 *
 *  Copyright(C) 2007,2008 Ixonos Plc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
 */

/*
 * Written by
 *   Jukka Honkela
 *   Yi Zheng
 *   Matti Katila
 *   Vesa Pikki
 *   Heikki Paajanen
 */


#include "dbus_phonebook_object.h"
#include "dbus_phonebook_glue.h"
#include "dbus_marshal.h"
#include <stdlib.h>
#include <stdio.h>

G_DEFINE_TYPE(DBusPhonebookObject, dbus_phonebook_object, G_TYPE_OBJECT)



static void
dbus_phonebook_object_class_init(DBusPhonebookObjectClass *klass)
{
        /* Make the local system aware of our kind */


}


static void dbus_phonebook_object_init(DBusPhonebookObject *gsm)
{
        dbus_g_object_type_install_info (DBUS_PHONEBOOK_TYPE_OBJECT,
                                         &dbus_glib_smartphone_phonebook_object_info);
}


void dbus_phonebook_object_finalize(GObject *obj)
{
        //G_OBJECT_CLASS (parent_class)->finalize (obj);
        //DBusPhonebookObject *gsm = (DBusPhonebookObject *) obj;
}




void gsmd_dbus_phonebook_uninitialize(DBusPhonebookObject *obj)
{
        g_object_unref(obj);
}



/********************************Methods***************************************/
gboolean smartphone_phonebook_get_storage_spaces(DBusPhonebookObject *obj,
                DBusGMethodInvocation *method_context)
{
        obj->phonebook->get_storage_spaces(obj->phonebook,
                                           method_context);
        return TRUE;
}

gboolean smartphone_phonebook_get_storage_info(DBusPhonebookObject *obj,
                DBusGMethodInvocation *method_context)
{
        obj->phonebook->get_storage_info(obj->phonebook,
                                         method_context);
        return TRUE;
}

gboolean smartphone_phonebook_get_default_storage_space(DBusPhonebookObject *obj,
                DBusGMethodInvocation *method_context)
{
        obj->phonebook->get_default_storage_space(obj->phonebook,
                        method_context);
        return TRUE;
}

gboolean smartphone_phonebook_set_default_storage_space(DBusPhonebookObject *obj,
                const gchar *space,
                DBusGMethodInvocation *method_context)
{
        obj->phonebook->set_default_storage_space(obj->phonebook,
                        method_context,space);
        return TRUE;
}

gboolean smartphone_phonebook_get_index_boundaries(DBusPhonebookObject *obj,
                DBusGMethodInvocation *method_context)
{
        obj->phonebook->get_index_boundaries(obj->phonebook,
                                             method_context);
        return TRUE;
}

gboolean smartphone_phonebook_delete_entry(DBusPhonebookObject *obj,
                const gint index,
                DBusGMethodInvocation *method_context)
{
        obj->phonebook->delete_entry(obj->phonebook,
                                     method_context,
                                     index);
        return TRUE;
}

gboolean smartphone_phonebook_store_entry(DBusPhonebookObject *obj,
                const gint index,
                const gint type,
                const gchar *name,
                const gchar *contents,
                DBusGMethodInvocation *method_context)
{
        obj->phonebook->store_entry(obj->phonebook,
                                    method_context,
                                    index,
                                    type,
                                    name,
                                    contents);
        return TRUE;
}

gboolean smartphone_phonebook_retrieve_entry(DBusPhonebookObject *obj,
                const gint index,
                DBusGMethodInvocation *method_context)
{
        obj->phonebook->retrieve_entry(obj->phonebook,
                                       method_context,
                                       index);
        return TRUE;
}


/********************************Method replies********************************/
void smartphone_phonebook_get_storage_spaces_reply(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GArray *spaces)
{
        if (ipc_data) {
                char **result = g_new0(char *,spaces->len+1);
                int i=0;
                GString *line = NULL;

                for(i=0;i<spaces->len;i++) {
                    line = g_array_index(spaces,GString*,i);
                    if (line)
                        result[i] = line->str;
                }

                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,
                                     result);
                g_free(result); //text inside will be free'd by caller
        }
}

void smartphone_phonebook_get_storage_info_reply(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GArray *spaces)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,
                                     spaces);

}

void smartphone_phonebook_get_default_storage_space_reply(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                const char *space)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,
                                     space);

}

void smartphone_phonebook_set_default_storage_space_reply(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);

}

void smartphone_phonebook_get_index_boundaries_reply(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                const int lowest_index,
                const int highest_index)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,
                                     lowest_index,
                                     highest_index);

}

void smartphone_phonebook_delete_entry_reply(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);

}

void smartphone_phonebook_store_entry_reply(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);

}

void smartphone_phonebook_retrieve_entry_reply(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                const int type,
                const char *name,
                const char *contents)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,
                                     type,
                                     name,
                                     contents);

}


/********************************Method error replies**************************/
void smartphone_phonebook_get_storage_spaces_error(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,error);
}

void smartphone_phonebook_get_storage_info_error(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,error);
}

void smartphone_phonebook_get_default_storage_space_error(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,error);
}

void smartphone_phonebook_set_default_storage_space_error(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,error);
}

void smartphone_phonebook_get_index_boundaries_error(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,error);
}

void smartphone_phonebook_delete_entry_error(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,error);
}

void smartphone_phonebook_store_entry_error(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,error);
}

void smartphone_phonebook_retrieve_entry_error(PhonebookIPCInterface *phonebook_ipc,
                gpointer ipc_data,
                GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,error);
}



DBusPhonebookObject *gsmd_dbus_phonebook_initialize (PhonebookIPCInterface *phonebook_ipc,
                PhonebookInterface *phonebook)
{
        DBusPhonebookObject *obj = g_object_new (DBUS_PHONEBOOK_TYPE_OBJECT, NULL);
        phonebook_ipc->priv = obj;
        obj->phonebook = phonebook;

        phonebook_ipc->get_storage_spaces_reply = &smartphone_phonebook_get_storage_spaces_reply;
        phonebook_ipc->get_storage_info_reply = &smartphone_phonebook_get_storage_info_reply;
        phonebook_ipc->get_default_storage_space_reply = &smartphone_phonebook_get_default_storage_space_reply;
        phonebook_ipc->set_default_storage_space_reply = &smartphone_phonebook_set_default_storage_space_reply;
        phonebook_ipc->get_index_boundaries_reply = &smartphone_phonebook_get_index_boundaries_reply;
        phonebook_ipc->delete_entry_reply = &smartphone_phonebook_delete_entry_reply;
        phonebook_ipc->store_entry_reply = &smartphone_phonebook_store_entry_reply;
        phonebook_ipc->retrieve_entry_reply = &smartphone_phonebook_retrieve_entry_reply;

        phonebook_ipc->get_storage_spaces_error = &smartphone_phonebook_get_storage_spaces_error;
        phonebook_ipc->get_storage_info_error = &smartphone_phonebook_get_storage_info_error;
        phonebook_ipc->get_default_storage_space_error = &smartphone_phonebook_get_default_storage_space_error;
        phonebook_ipc->set_default_storage_space_error = &smartphone_phonebook_set_default_storage_space_error;
        phonebook_ipc->get_index_boundaries_error = &smartphone_phonebook_get_index_boundaries_error;
        phonebook_ipc->delete_entry_error = &smartphone_phonebook_delete_entry_error;
        phonebook_ipc->store_entry_error = &smartphone_phonebook_store_entry_error;
        phonebook_ipc->retrieve_entry_error = &smartphone_phonebook_retrieve_entry_error;

        return obj;
}

