/*
 *  dbus_phonebook_object.h
 *
 *  Copyright(C) 2007,2008 Ixonos Plc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
 */

/*
 * Written by
 *   Jukka Honkela
 *   Yi Zheng
 *   Matti Katila
 *   Vesa Pikki
 *   Heikki Paajanen
 */


#ifndef DBUS_PHONEBOOK_OBJECT_H
#define DBUS_PHONEBOOK_OBJECT_H


#include <glib-object.h>
#include "dbus_objects.h"
#include "phonebook_interface.h"


typedef struct DBusPhonebookObject DBusPhonebookObject;
typedef struct DBusPhonebookObjectClass DBusPhonebookObjectClass;

GType dbus_phonebook_object_get_type (void);


struct DBusPhonebookObject {
        GObject parent;
        PhonebookInterface *phonebook;
};

struct DBusPhonebookObjectClass {
        GObjectClass parent;
};

/* Macros that are needed for making the proper GObject */
#define DBUS_PHONEBOOK_TYPE_OBJECT              (dbus_phonebook_object_get_type ())
#define DBUS_PHONEBOOK_OBJECT(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), DBUS_PHONEBOOK_TYPE_OBJECT, DBusPhonebookObject))
#define DBUS_PHONEBOOK_OBJECT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), DBUS_PHONEBOOK_TYPE_OBJECT, DBusPhonebookObjectClass))
#define DBUS_IS_PHONEBOOK_OBJECT(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), DBUS_PHONEBOOK_TYPE_OBJECT))
#define DBUS_IS_PHONEBOOK_OBJECT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), DBUS_PHONEBOOK_TYPE_OBJECT))
#define DBUS_PHONEBOOK_OBJECT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), DBUS_PHONEBOOK_TYPE_OBJECT, DBusPhonebookObjectClass))

DBusPhonebookObject *gsmd_dbus_phonebook_initialize (PhonebookIPCInterface *phonebook_ipc,
                                                    PhonebookInterface *phonebook);
void gsmd_dbus_phonebook_uninitialize(DBusPhonebookObject *obj);

//Methods
gboolean smartphone_phonebook_get_storage_spaces(DBusPhonebookObject *obj, DBusGMethodInvocation *method_context);
gboolean smartphone_phonebook_get_storage_info(DBusPhonebookObject *obj, DBusGMethodInvocation *method_context);
gboolean smartphone_phonebook_get_default_storage_space(DBusPhonebookObject *obj, DBusGMethodInvocation *method_context);
gboolean smartphone_phonebook_set_default_storage_space(DBusPhonebookObject *obj, const gchar *space,DBusGMethodInvocation *method_context);
gboolean smartphone_phonebook_get_index_boundaries(DBusPhonebookObject *obj, DBusGMethodInvocation *method_context);
gboolean smartphone_phonebook_delete_entry(DBusPhonebookObject *obj, const gint index, DBusGMethodInvocation *method_context);
gboolean smartphone_phonebook_store_entry(DBusPhonebookObject *obj, const gint index, const gint type, const gchar *name, const gchar *contents, DBusGMethodInvocation *method_context);
gboolean smartphone_phonebook_retrieve_entry(DBusPhonebookObject *obj, const gint index, DBusGMethodInvocation *method_context);







#endif
