/*
 *  dbus_sim_object.c
 *
 *  Copyright(C) 2007,2008 Ixonos Plc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
 */

/*
 * Written by
 *   Jukka Honkela
 *   Yi Zheng
 *   Matti Katila
 *   Vesa Pikki
 *   Heikki Paajanen
 */


#include "dbus_sim_object.h"
#include "dbus_sim_glue.h"
#include "dbus_marshal.h"
#include "sim_interface.h"
#include <stdlib.h>
#include <stdio.h>

G_DEFINE_TYPE(DBusSimObject, dbus_sim_object, G_TYPE_OBJECT)



static void
dbus_sim_object_class_init(DBusSimObjectClass *klass)
{
        /* Make the local system aware of our kind */

        /** Register our signals so that they can be emitted */

        klass->auth_status =
                g_signal_new ("auth_status",
                              G_OBJECT_CLASS_TYPE (klass),
                              G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                              0, NULL, NULL, g_cclosure_marshal_VOID__STRING,
                              G_TYPE_NONE, 1, G_TYPE_STRING);
}


static void dbus_sim_object_init(DBusSimObject *gsm)
{
        dbus_g_object_type_install_info (DBUS_SIM_TYPE_OBJECT,
                                         &dbus_glib_smartphone_sim_object_info);
}


void dbus_sim_object_finalize(GObject *obj)
{
        //G_OBJECT_CLASS (parent_class)->finalize (obj);
        //DBusSimObject *gsm = (DBusSimObject *) obj;
}




void gsmd_dbus_sim_uninitialize(DBusSimObject *obj)
{
        g_object_unref(obj);
}



/*********************Signal functions***********************/
void smartphone_sim_auth_status (SIMIPCInterface *sim_ipc, const gchar *status)
{
        g_debug("%s : %s",__func__,status);
        DBusSimObject *obj = (DBusSimObject*)sim_ipc->priv;
        g_signal_emit (obj,
                       DBUS_SIM_OBJECT_GET_CLASS(obj)->auth_status,
                       0, status);
}


/*********************Methods***********************/
gboolean smartphone_sim_get_messagebook_info(DBusSimObject *obj,
                                             DBusGMethodInvocation *method_context)
{
        obj->sim->get_messagebook_info( obj->sim,
                                        method_context);
        return TRUE;
}

gboolean smartphone_sim_store_message(DBusSimObject *obj,
                                      const gchar *recipient_number,
                                      const gchar *contents,
                                      DBusGMethodInvocation *method_context)
{
        obj->sim->store_message( obj->sim,
                                 method_context,
                                 recipient_number,
                                 contents);
        return TRUE;
}

gboolean smartphone_sim_delete_message(DBusSimObject *obj,
                                       const gint index,
                                       DBusGMethodInvocation *method_context)
{
        obj->sim->delete_message(obj->sim,method_context,index);
        return TRUE;
}

gboolean smartphone_sim_list_messages(DBusSimObject *obj,
                                      const gchar *status,
                                      DBusGMethodInvocation *method_context)
{
        obj->sim->list_messages(obj->sim,method_context,status);
        return TRUE;
}

gboolean smartphone_sim_send_message(DBusSimObject *obj,
                                     gint index,
                                     gboolean want_report,
                                     DBusGMethodInvocation *method_context)
{
        obj->sim->send_message(obj->sim,method_context,index,want_report);
        return TRUE;
}

gboolean smartphone_sim_retrieve_message(DBusSimObject *obj,
                                         const gint number,
                                         DBusGMethodInvocation *method_context)
{
        g_debug("%s : %d",__func__,number);
        obj->sim->retrieve_message(obj->sim,method_context,number);
        return TRUE;
}

gboolean smartphone_sim_get_country_code(DBusSimObject *obj,
                                         DBusGMethodInvocation *method_context)
{
        obj->sim->get_country_code(obj->sim,
                                   method_context);

        return TRUE;
}


gboolean smartphone_sim_get_subscriber_numbers(DBusSimObject *obj,
                                               DBusGMethodInvocation *method_context)
{
        obj->sim->get_subscriber_numbers(obj->sim,
                                         method_context);
        return TRUE;
}


gboolean smartphone_sim_get_service_center_number(DBusSimObject *obj,
                                                  DBusGMethodInvocation *method_context)
{
        obj->sim->get_service_center_number(obj->sim,method_context);
        return TRUE;
}

gboolean smartphone_sim_set_service_center_number(DBusSimObject *obj,
                                                  const gchar *number,
                                                  DBusGMethodInvocation *method_context)
{
        obj->sim->set_service_center_number( obj->sim,
                                             method_context,
                                             number);
        return TRUE;
}


gboolean smartphone_sim_get_auth_status(DBusSimObject *obj,
                                        DBusGMethodInvocation *method_context)
{
        obj->sim->get_auth_status(obj->sim,method_context);

        return TRUE;
}


gboolean smartphone_sim_send_auth_code(DBusSimObject *obj,
                                       const gchar *pin,
                                       DBusGMethodInvocation *method_context)
{
        obj->sim->send_auth_code(obj->sim,method_context,pin);
        return TRUE;
}

gboolean smartphone_sim_unlock(DBusSimObject *obj,
                               const gchar *puk,
                               const gchar *new_pin,
                               DBusGMethodInvocation *method_context)
{
        obj->sim->unlock(obj->sim,method_context,puk,new_pin);
        return TRUE;
}

gboolean smartphone_sim_change_auth_code(DBusSimObject *obj,
                                         const gchar *old_pin,
                                         const gchar *new_pin,
                                         DBusGMethodInvocation *method_context)
{
        obj->sim->change_auth_code(  obj->sim,
                                     method_context,
                                     old_pin,
                                     new_pin);
        return TRUE;
}

gboolean smartphone_sim_get_imsi(DBusSimObject *obj,
                                 DBusGMethodInvocation *method_context)
{
        obj->sim->get_imsi(obj->sim,method_context);
        return TRUE;
}

gboolean smartphone_sim_get_phonebook_info(DBusSimObject *obj,
                                           DBusGMethodInvocation *method_context)
{
        obj->sim->get_phonebook_info(obj->sim,method_context);
        return TRUE;
}

gboolean smartphone_sim_delete_entry(DBusSimObject *obj,
                                     DBusGMethodInvocation *method_context,
                                     gint index)
{
        obj->sim->delete_entry(obj->sim,method_context,index);
        return TRUE;
}


gboolean smartphone_sim_store_entry(DBusSimObject *obj,
                                    DBusGMethodInvocation *method_context,
                                    const gchar *recipient_number,
                                    const gchar *contents,
                                    gint index)
{
        obj->sim->store_entry(obj->sim,
                              method_context,
                              recipient_number,
                              contents,
                              index);
        return TRUE;
}

gboolean smartphone_sim_retrieve_entry(DBusSimObject *obj,
                                       DBusGMethodInvocation *method_context,
                                       gint index)
{
        obj->sim->retrieve_entry(obj->sim,
                                 method_context,
                                 index);
        return TRUE;
}

/****************************Method replies************************************/
void smartphone_sim_get_phonebook_info_reply(SIMIPCInterface *sim_ipc,
                                             gpointer ipc_data,
                                             GHashTable *info)
{
        //TODO dbus_g_method_return most likely doesn't want GHashTable, check&fix
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,info);
}


void smartphone_sim_get_messagebook_info_reply(SIMIPCInterface *sim_ipc,
                                               gpointer ipc_data,
                                               GHashTable *info)
{
        //TODO dbus_g_method_return most likely doesn't want GHashTable, check&fix
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,info);
}

void smartphone_sim_store_message_reply(SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data,
                                        const int index)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,index);
}

void smartphone_sim_retrieve_message_reply(SIMIPCInterface *sim_ipc,
                                           gpointer ipc_data,
                                           const char *sender_number,
                                           const char *content,
                                           gint time)
{
        g_debug("%s: %s to %s",__func__,content,sender_number);
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,
                                     sender_number,
                                     content,
                                     time);
}


void smartphone_sim_get_subscriber_numbers_reply(SIMIPCInterface *sim_ipc,
                                                 gpointer ipc_data,
                                                 const char **numbers)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,
                                     numbers);
}


void smartphone_sim_get_auth_status_reply(SIMIPCInterface *sim_ipc,
                                          gpointer ipc_data,
                                          const char *status)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,status);
}

void smartphone_sim_send_auth_code_reply(SIMIPCInterface *sim_ipc,
                                         gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);
}

void smartphone_sim_delete_message_reply(SIMIPCInterface *sim_ipc,
                                         gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);
}

void smartphone_sim_send_message_reply(SIMIPCInterface *sim_ipc,
                                       gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);
}

void smartphone_sim_list_messages_reply(SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data,
                                        GArray *indices)
{
        gint i=0;
        g_debug("%s: Message indices: ",__func__);
        for (i=0;i<indices->len;i++) {
                g_debug("%s: %d ",__func__,g_array_index(indices,int,i));
        }
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,indices);
}

void smartphone_sim_unlock_reply(SIMIPCInterface *sim_ipc,
                                 gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);
}

void smartphone_sim_change_auth_code_reply(SIMIPCInterface *sim_ipc,
                                           gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);
}

void smartphone_sim_get_imsi_reply(SIMIPCInterface *sim_ipc,
                                   gpointer ipc_data,
                                   const char *imsi)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,imsi);
}

void smartphone_sim_get_service_center_number_reply(SIMIPCInterface *sim_ipc,
                                                    gpointer ipc_data,
                                                    const char *number)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,number);
}

void smartphone_sim_set_service_center_number_reply(SIMIPCInterface *sim_ipc,
                                                    gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);
}

void smartphone_sim_delete_entry_reply(SIMIPCInterface *sim_ipc,
                                       gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);
}

void smartphone_sim_store_entry_reply(SIMIPCInterface *sim_ipc,
                                      gpointer ipc_data)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data);
}

void smartphone_sim_retrieve_entry_reply(SIMIPCInterface *sim_ipc,
                                         gpointer ipc_data,
                                         const gchar *name,
                                         const gchar *number)
{
        if (ipc_data)
                dbus_g_method_return((DBusGMethodInvocation*) ipc_data,
                                     name,
                                     number);
}

/****************************Method error replies******************************/
void smartphone_sim_get_phonebook_info_error(SIMIPCInterface *sim_ipc,
                                             gpointer ipc_data,
                                             GError *error)
{
        g_debug("%s", __func__);
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_delete_message_error(SIMIPCInterface *sim_ipc,
                                         gpointer ipc_data,
                                         GError *error)
{
        g_debug("%s", __func__);
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_send_message_error(SIMIPCInterface *sim_ipc,
                                       gpointer ipc_data,
                                       GError *error)
{
        g_debug("%s", __func__);
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_list_messages_error(SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data,
                                        GError *error)
{
        g_debug("%s", __func__);
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_get_messagebook_info_error(SIMIPCInterface *sim_ipc,
                                               gpointer ipc_data,
                                               GError *error)
{
        g_debug("%s", __func__);
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_get_subscriber_numbers_error(SIMIPCInterface *sim_ipc,
                                                 gpointer ipc_data,
                                                 GError *error)
{
        g_debug("%s", __func__);
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}


void smartphone_sim_get_auth_status_error(SIMIPCInterface *sim_ipc,
                                          gpointer ipc_data,
                                          GError *error)
{
        g_debug("%s", __func__);
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_send_auth_code_error(SIMIPCInterface *sim_ipc,
                                         gpointer ipc_data,
                                         GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_unlock_error(SIMIPCInterface *sim_ipc,
                                 gpointer ipc_data,
                                 GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_change_auth_code_error(SIMIPCInterface *sim_ipc,
                                           gpointer ipc_data,
                                           GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_get_imsi_error(SIMIPCInterface *sim_ipc,
                                   gpointer ipc_data,
                                   GError *error)
{
        if (ipc_data) {
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
        }
}

void smartphone_sim_get_service_center_number_error(SIMIPCInterface *sim_ipc,
                                                    gpointer ipc_data,
                                                    GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_set_service_center_number_error(SIMIPCInterface *sim_ipc,
                                                    gpointer ipc_data,
                                                    GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_store_message_error(SIMIPCInterface *sim_ipc,
                                        gpointer ipc_data,
                                        GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_retrieve_message_error(SIMIPCInterface *sim_ipc,
                                           gpointer ipc_data,
                                           GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_delete_entry_error(SIMIPCInterface *sim_ipc,
                                       gpointer ipc_data,
                                       GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_store_entry_error(SIMIPCInterface *sim_ipc,
                                      gpointer ipc_data,
                                      GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}

void smartphone_sim_retrieve_entry_error(SIMIPCInterface *sim_ipc,
                                         gpointer ipc_data,
                                         GError *error)
{
        if (ipc_data)
                dbus_g_method_return_error((DBusGMethodInvocation*) ipc_data,
                                           error);
}




DBusSimObject *gsmd_dbus_sim_initialize (SIMIPCInterface *sim_ipc,
                                         SIMInterface *sim)
{
        DBusSimObject *obj = g_object_new (DBUS_SIM_TYPE_OBJECT, NULL);
        sim_ipc->priv = obj;
        obj->sim = sim;

        //Signal
        sim_ipc->auth_status = &smartphone_sim_auth_status;

        //Method replies
        sim_ipc->get_imsi_reply = &smartphone_sim_get_imsi_reply;
        sim_ipc->change_auth_code_reply = &smartphone_sim_change_auth_code_reply;
        sim_ipc->unlock_reply = &smartphone_sim_unlock_reply;
        sim_ipc->send_auth_code_reply = &smartphone_sim_send_auth_code_reply;
        sim_ipc->get_auth_status_reply = &smartphone_sim_get_auth_status_reply;
        sim_ipc->get_service_center_number_reply = &smartphone_sim_get_service_center_number_reply;
        sim_ipc->set_service_center_number_reply = &smartphone_sim_set_service_center_number_reply;
        sim_ipc->get_subscriber_numbers_reply = &smartphone_sim_get_subscriber_numbers_reply;
        sim_ipc->store_message_reply = &smartphone_sim_store_message_reply;
        sim_ipc->retrieve_message_reply = &smartphone_sim_retrieve_message_reply;
        sim_ipc->get_phonebook_info_reply = &smartphone_sim_get_phonebook_info_reply;
        sim_ipc->get_messagebook_info_reply = &smartphone_sim_get_phonebook_info_reply;
        sim_ipc->delete_entry_reply = &smartphone_sim_delete_entry_reply;
        sim_ipc->store_entry_reply = &smartphone_sim_store_entry_reply;
        sim_ipc->retrieve_entry_reply = &smartphone_sim_retrieve_entry_reply;
        sim_ipc->delete_message_reply = &smartphone_sim_delete_message_reply;
        sim_ipc->send_message_reply = &smartphone_sim_send_message_reply;
        sim_ipc->list_messages_reply = &smartphone_sim_list_messages_reply;


        //Method errors
        sim_ipc->get_imsi_error = &smartphone_sim_get_imsi_error;
        sim_ipc->change_auth_code_error = &smartphone_sim_change_auth_code_error;
        sim_ipc->unlock_error = &smartphone_sim_unlock_error;
        sim_ipc->send_auth_code_error = &smartphone_sim_send_auth_code_error;
        sim_ipc->get_auth_status_error = &smartphone_sim_get_auth_status_error;
        sim_ipc->get_service_center_number_error = &smartphone_sim_get_service_center_number_error;
        sim_ipc->set_service_center_number_error = &smartphone_sim_set_service_center_number_error;
        sim_ipc->get_subscriber_numbers_error = &smartphone_sim_get_subscriber_numbers_error;
        sim_ipc->store_message_error = &smartphone_sim_store_message_error;
        sim_ipc->retrieve_message_error = &smartphone_sim_retrieve_message_error;
        sim_ipc->get_phonebook_info_error = &smartphone_sim_get_phonebook_info_error;
        sim_ipc->get_messagebook_info_error = &smartphone_sim_get_phonebook_info_error;
        sim_ipc->delete_entry_error = &smartphone_sim_delete_entry_error;
        sim_ipc->store_entry_error = &smartphone_sim_store_entry_error;
        sim_ipc->retrieve_entry_error = &smartphone_sim_retrieve_entry_error;
        sim_ipc->delete_message_error = &smartphone_sim_delete_message_error;
        sim_ipc->send_message_error = &smartphone_sim_send_message_error;
        sim_ipc->list_messages_error = &smartphone_sim_list_messages_error;


        return obj;
}
