# Copyright(C) 2007,2008 Ixonos Plc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Boston, MA 02111.

# Written by Matti Katila, Yi Zheng, Vesa Pikki and Heikki Paajanen
from ipc_interface import IPCInterface
import threading, time
import pyfreesmartphone
import gobject

def fso_callback(data,error,*arg):
    if error:
        data.ErrorHandler(error)
    else:
        data.ReplyHandler(arg)

def call_status_signal(data, id, status, properties):
    data.AddSignal(("CallStatus",id, status, properties))

def sms_incoming_message_signal(data,index):
    data.message_index = index
    data.AddSignal(("IncomingMessage",index))
def sms_message_sent_signal(data,success,reason):
    data.AddSignal(("MessageSent",success,reason))

def sim_auth_status_signal(data, id, status):
    data.AddSignal(("AuthStatus",status))


class FSOGMainLoopThread(threading.Thread):
    def __init__ (self):
        threading.Thread.__init__(self)
        self.loop = None

    def stop(self):
        if self.loop:
            self.loop.quit()

    def run(self):
        self.loop = gobject.MainLoop()
        self.loop.run()
        self.loop = None


'''
def dial_callback(id, error, data):
    print str(data)
    if error:
        print "Got error"
        data.error = True
        data.ErrorHandler(error)
    else:
        data.call_id = int(id)
        print "Got call id " + str(id)
        data.ReplyHandler(id)
    data.reply_event.set()
'''
class FSOInterface(IPCInterface):
    def Initialize(self):
        IPCInterface.Initialize(self)
        self.mainloop_thread = None
        self.start_thread()

        self.fs_call = pyfreesmartphone.PyFreeSmartphoneCall()
        self.fs_sms = pyfreesmartphone.PyFreeSmartphoneSMS()
        self.fs_sim = pyfreesmartphone.PyFreeSmartphoneSIM()
        self.fs_device = pyfreesmartphone.PyFreeSmartphoneDevice()

        self.fs_sim.auth_status_signal(sim_auth_status_signal, self)
        self.fs_call.status_signal(call_status_signal, self)
        self.fs_sms.incoming_message_signal(sms_incoming_message_signal, self)
        self.fs_sms.message_sent_signal(sms_message_sent_signal, self)
        #self.fs_sim.auth_status_signal(sms_message_sent_signal, self)
        
    def DeInitialize(self):
        IPCInterface.DeInitialize(self)
        self.stop_thread()
        self.fs_call.status_signal_remove()
        self.fs_sim.auth_status_signal_remove()
        self.fs_sms.incoming_message_signal_remove()
        self.fs_sms.message_sent_signal_remove()
        #self.fs.deinitialize()
        
        
    
    #Call interface
    def Call_Emergency(self):
        pass
    def Call_Activate(self, id):
        self.fs_call.activate(id, fso_callback, self)
    def Call_Release(self,message,id):
        self.fs_call.release(message, id, fso_callback, self)
    def Call_Initiate(self,number,type):
        self.fs_call.initiate(number,fso_callback, self)
        
    def Call_GetStatus(self,id):
        self.fs_call.get_call_status(id,fso_callback, self)
    def Call_SendDtmf(self):
        pass
    def Call_SetDtmfDuration(self,mode):
        pass
    def Call_GetDtmfDuration(self):
        pass

    #Device interface
    def Device_GetInfo(self):
        pass
    def Device_SetAntennaPower(self,antenna_power):
        pass
    def Device_GetAntennaPower(self):
        self.fs_device.get_antenna_power(fso_callback, self)
    def Device_GetFeatures(self):
        pass

    #Network interface
    def Network_Register(self):
        pass
    def Network_GetStatus(self):
        pass
    def Network_ListProviders(self):
        pass
    def Network_RegisterWithProvider(self,index):
        pass
    def Network_GetSubscriberNumbers(self):
        pass
    def Network_GetCountryCode(self):
        pass
    def Network_GetHomeCountryCode(self):
        pass

    #SMS interface
    def SMS_GetServiceBearer(self):
        self.fs_sms.get_service_bearer(fso_callback, self)
    def SMS_SetServiceBearer(self,mode):
        self.fs_sms.set_service_bearer(mode,fso_callback, self)
    def SMS_GetServiceCenterNumber(self):
        self.fs_sms.get_service_center_number(fso_callback, self)
    def SMS_SetServiceCenterNumber(self,number):
        self.fs_sms.set_service_center_number(number,fso_callback, self)
    def SMS_GetStorageSpaces(self):
        pass
    def SMS_GetDefaultStorageSpace(self):
        pass
    def SMS_SetDefaultStorageSpace(self,space):
        pass
    def SMS_ListMessages(self):
        self.fs_sms.list_messages(fso_callback, self)
    def SMS_DeleteMessage(self,index):
        self.fs_sms.delete_message(index,fso_callback, self)
    def SMS_SendMessage(self,index,want_report):
        self.fs_sms.send_message(index,want_report,fso_callback, self)
    def SMS_StoreMessage(self,recipient_number,contents):
        self.fs_sms.store_message(recipient_number,contents,fso_callback, self)
    def SMS_RetrieveMessage(self,number):
        self.fs_sms.retrieve_message(number,fso_callback, self)

    #SIM interface
    def SIM_GetAuthStatus(self):
        pass
    def SIM_SendAuthCode(self,pin):
        pass
    def SIM_Unlock(self,puk,new_pin):
        pass
    def SIM_ChangeAuthCode(self,old_pin,new_pin):
        pass
    def SIM_GetImsi(self):
        pass

    #Phonebook interface
    def Phonebook_GetStorageSpaces(self):
        pass
    def Phonebook_GetStorageInfo(self):
        pass
    def Phonebook_GetDefaultStorageSpace(self):
        pass
    def Phonebook_SetDefaultStorageSpace(self,space):
        pass
    def Phonebook_GetIndexBoundaries(self):
        pass
    def Phonebook_DeleteEntry(self,index):
        pass
    def Phonebook_StoreEntry(self,index,type,name,contents):
        pass
    def Phonebook_RetrieveEntry(self,index):
        pass

    #PDP interface
    def PDP_ListGprsClasses(self):
        pass
    def PDP_SelectGprsClass(self,klass):
        pass
    def PDP_Activate(self,index):
        pass
    def PDP_Deactivate(self,index):
        pass
    def PDP_SelectContext(self,context):
        pass
    def PDP_AddContext(self,context):
        pass
    def PDP_DeleteContext(self,index):
        pass
    def PDP_ListContexts(self):
        pass

    #Mainloop
    def stop_thread(self):
        if not self.mainloop_thread:
            return
        self.mainloop_thread.stop()
        while self.mainloop_thread.loop != None:
            time.sleep(0.1)
        self.mainloop_thread = None


    def start_thread(self):
        self.mainloop_thread = FSOGMainLoopThread()
        self.mainloop_thread.start()




