#
# SIM cdefs
#
cdef extern from "../libfreesmartphone/freesmartphone-sim.h":
    ctypedef void (*FSGSMSIMGetAuthStatusReplyFunc) (gchar *status,
                                                GError *error,
                                                gpointer userdata )
    ctypedef void (*FSGSMSIMReplyFunc) (GError *error,
                                   gpointer userdata )
    ctypedef void (*FSGSMSIMGetImsiReplyFunc) (gchar *imsi,
                                          GError *error,
                                          gpointer userdata )
    gboolean fso_gsm_sim_get_imsi(FreeSmartphone *fs,
                              FSGSMSIMGetImsiReplyFunc cb,
                              gpointer userdata)
    gboolean fso_gsm_sim_change_auth_code(FreeSmartphone *fs,
                                      gchar *old_pin,
                                      gchar *new_pin,
                                      FSGSMSIMReplyFunc cb,
                                      gpointer userdata)
    gboolean fso_gsm_sim_unlock(FreeSmartphone *fs,
                            gchar *puk,
                            gchar *new_pin,
                            FSGSMSIMReplyFunc cb,
                            gpointer userdata)
    gboolean fso_gsm_sim_send_auth_code(FreeSmartphone *fs,
                                    gchar *pin,
                                    FSGSMSIMReplyFunc cb,
                                    gpointer userdata)
    gboolean fso_gsm_sim_get_auth_status(FreeSmartphone *fs,
                                     FSGSMSIMGetAuthStatusReplyFunc cb,
                                     gpointer userdata)
    ctypedef void (*FSGSMSIMAuthStatusSignalFunc) (gchar *status,
                                              gpointer userdata)
    gboolean fso_gsm_sim_auth_status_signal_remove(FreeSmartphone *fs,
                                               FSGSMSIMAuthStatusSignalFunc cb,
                                               gpointer userdata)
    gboolean fso_gsm_sim_auth_status_signal(FreeSmartphone *fs,
                                        FSGSMSIMAuthStatusSignalFunc cb,
                                        gpointer userdata)

#
# SIM Handlers
#

cdef void auth_status_signal(gchar *status, gpointer userdata) with gil:
    print "SIM auth status signal"
    reply = <object>userdata
    reply_cb = reply.get_cb()
    reply_data = reply.get_data()
    status_str = PyString_FromStringAndSize(status, strlen(status))
    reply_cb(reply_data, id, status_str)

#
# PyFreeSmartPhoneSIM class
#

cdef class PyFreeSmartphoneSIM:
    cdef FreeSmartphone *fs

    cdef object auth_status

    def __new__(self):
        self.fs = fso_init()
    def deinitialize(self):
        fso_free(self.fs)

    def auth_status_signal(self, cb,data=None):
        self.auth_status = ReplyData(cb,data)
        Py_INCREF(self.auth_status)
        return fso_gsm_sim_auth_status_signal(self.fs, &auth_status_signal, <void*>self.auth_status)

    def auth_status_signal_remove(self):
        fso_gsm_sim_auth_status_signal_remove(self.fs, &auth_status_signal, <void*>self.auth_status)
        Py_DECREF(self.auth_status)
